#!/bin/bash
#
# Copyright (c) Microsoft Corporation.
#
# This script will
#   1.  Examine Azcmagent configuration
#   2.  Check if automatic upgrade is enabled
#   3.  Check if desired version is greater than existing agent version
#   4.  Check if automatic upgrade has already been attempted before
#   5.  Execute install_linux_azcmagent.sh to upgrade
#
# Note that this script is for Linux only

# Error codes used by azcmagent are in range of [0, 125].
# Installation scripts will use [127, 255]. Check install_azcmagent.ps1 for the codes used for Windows script.
# Upgrade script will use [257, 300]. Check azcmagent_check_updates.ps1 for the codes used for Windows script.
function log() {
  logBody="{\"subscriptionId\":\"${subscriptionId}\",\"resourceGroup\":\"${resourceGroup}\",\"resourceName\":\"${resourceName}\",\"tenantId\":\"${tenantId}\",\"location\":\"${location}\",\"correlationId\":\"${correlationId}\",\"operation\":\"upgrading\",\"messageType\":\"$1\",\"message\":\"$2\"}"

  his_endpoint=https://gbl.his.arc.azure.com
  if [ "${cloud}" = "AzureUSGovernment" ]; then
    his_endpoint=https://gbl.his.arc.azure.us
  elif [ "${cloud}" = "AzureChinaCloud" ]; then
    his_endpoint=https://gbl.his.arc.azure.cn
  fi

  if command -v wget &> /dev/null; then
    if [ -n "${proxy}" ]; then
        wget -qO- -e use_proxy=yes -e http_proxy=${proxy} --method=PUT --body-data="$logBody" ${his_endpoint}/log &> /dev/null || true
    else
        wget -qO- --method=PUT --body-data="$logBody" ${his_endpoint}/log &> /dev/null || true
    fi
  elif command -v curl &> /dev/null; then
    if [ -n "${proxy}" ]; then
        curl -s -X PUT --proxy ${proxy} -d "$logBody" ${his_endpoint}/log &> /dev/null || true
    else
        curl -s -X PUT -d "$logBody" ${his_endpoint}/log &> /dev/null || true
    fi
  fi
}

function versionCompare() {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i ver1=($1) ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

if [ "$(id -u)" -ne 0 ]; then
    log 257 "$0: Must be root to execute"
    exit 18
fi

show=$(azcmagent show --json)
if [ $? -ne 0 ]; then
    log 258 "$0: azcmagent show command failed"
    exit 1
fi

cloud=$(echo $show | grep -Eo '"cloud"[^,]*' | grep -Eo '[^: ]*$' | tr -d '"')
subscriptionId=$(echo $show | grep -Eo '"subscriptionId"[^,]*' | grep -Eo '[^: ]*$' | tr -d '"')
resourceGroup=$(echo $show | grep -Eo '"resourceGroup"[^,]*' | grep -Eo '[^: ]*$' | tr -d '"')
resourceName=$(echo $show | grep -Eo '"resourceName"[^,]*' | grep -Eo '[^: ]*$' | tr -d '"')
tenantId=$(echo $show | grep -Eo '"tenantId"[^,]*' | grep -Eo '[^: ]*$' | tr -d '"')
location=$(echo $show | grep -Eo '"location"[^,]*' | grep -Eo '[^: ]*$' | tr -d '"')
correlationId=$(echo $show | grep -Eo '"correlationId"[^,]*' | grep -Eo '[^: ]*$' | tr -d '"')
agentVersion=$(echo $show | grep -Eo '"agentVersion"[^,]*' | grep -Eo '[^: ]*$' | tr -d '"')

enabled=$(azcmagent config get automaticupgrade.enabled)
if [ $? -ne 0 ]; then
    log 259 "$0: unable to get automaticupgrade.enabled flag"
    exit 1
fi

if [ "$enabled" != "true" ]; then
    log 260 "$0: automaticupgrade.enabled not enabled"
    exit 0
fi

desiredVersion=$(azcmagent config get automaticupgrade.desiredversion)
if [ $? -ne 0 ]; then
    log 261 "$0: unable to get automaticupgrade.desiredversion"
    exit 1
fi

versionCompare $agentVersion $desiredVersion
if [ $? -ne 2 ]; then
    log 262 "$0: automaticupgrade.desiredversion ${desiredVersion} not greater than existing agent version ${agentVersion}"
    exit 0
fi

lastattempt=$(azcmagent config get automaticupgrade.lastattempt.status)
if [ $? -ne 0 ]; then
    log 263 "$0: unable to get automaticupgrade.lastattempt.status field"
    exit 1
fi

if [ "$lastattempt" = "success" ]; then
    log 264 "$0: Last attempt was successful, skipping upgrade"
    exit 0
fi

if [ "$lastattempt" = "failed" ]; then
    log 265 "$0: Last attempt failed, skipping upgrade"
    exit 0
fi

# do the upgrade using the on-boarding script
timestamp=$(date +%s)
outfile=/var/opt/azcmagent/log/install.log
/bin/bash /opt/azcmagent/bin/install_linux_azcmagent.sh -d $desiredVersion -o $outfile
if [ $? -ne 0 ]; then
    azcmagent config set automaticupgrade.lastattempt.status failed
    message=$(cat $outfile | grep -Eo '"message"[^,]*' | sed -nr 's/"message": (.*)/\1/p' | tr -d '"')
    azcmagent config set automaticupgrade.lastattempt.message "$message"
else
    azcmagent config set automaticupgrade.lastattempt.status success
    log 300 "$0: Upgrade was successful"
fi

azcmagent config set automaticupgrade.lastattempt.desiredVersion $desiredVersion
azcmagent config set automaticupgrade.lastattempt.timestamp $timestamp